// Client.cpp
#include "Client.h"
#include "Packet.h"
#include "Utils.h"
#include <winsock2.h>
#include <ws2tcpip.h>
#include <iostream>
#include <chrono>
#include <thread>
#include <string>

void sendAuthPacket(SOCKET sock, const std::string &username, const std::string &password) {
    Packet authPacket;
    authPacket.type = AUTH_REQUEST;  // Ensure AUTH_REQUEST is defined in your PacketType enum.
    authPacket.data = username + ":" + password;
    if(!sendPacket(sock, authPacket)) {
        std::cerr << "Failed to send auth packet." << std::endl;
    }
}

void runClient(const std::string& host, const std::string& portStr) {
    int port = std::stoi(portStr);
    SOCKET sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if(sock == INVALID_SOCKET) {
        std::cerr << "Failed to create socket." << std::endl;
        return;
    }
    
    sockaddr_in serverAddr;
    serverAddr.sin_family = AF_INET;
    inet_pton(AF_INET, host.c_str(), &serverAddr.sin_addr);
    serverAddr.sin_port = htons(port);
    
    if(connect(sock, reinterpret_cast<sockaddr*>(&serverAddr), sizeof(serverAddr)) == SOCKET_ERROR) {
        std::cerr << "Connect failed." << std::endl;
        closesocket(sock);
        return;
    }
    
    std::cout << "Connected to server." << std::endl;
    
    // Prompt for authentication details.
    std::string username, password;
    std::cout << "Enter username: ";
    std::getline(std::cin, username);
    std::cout << "Enter password: ";
    std::getline(std::cin, password);
    
    // Send authentication packet.
    sendAuthPacket(sock, username, password);
    
    Packet joinPacket;
    joinPacket.type = JOIN;
    joinPacket.data = "I have joined.";
    if(!sendPacket(sock, joinPacket)) {
        std::cerr << "Failed to send join packet." << std::endl;
        closesocket(sock);
        return;
    }
    
    // Main loop to receive packets from the server.
    Packet packet;
    while(receivePacket(sock, packet)) {
        switch(packet.type) {
            case HEARTBEAT: {
                try {
                    long long serverTimestamp = std::stoll(packet.data);
                    using namespace std::chrono;
                    auto now = system_clock::now();
                    auto msNow = duration_cast<milliseconds>(now.time_since_epoch()).count();
                    long long ping = msNow - serverTimestamp;
                    std::cout << "Heartbeat: " << ping << " ms" << std::endl;
                } catch(...) {
                }
                break;
            }
            case JOIN:
                std::cout << "Connect: " << packet.data << std::endl;
                break;
            case DISCONNECT:
                std::cout << "Disconnect: " << packet.data << std::endl;
                break;
            case AUTH_RESPONSE:
                std::cout << "Auth Response: " << packet.data << std::endl;
                break;
            default:
                std::cout << "Unknown packet type: " << static_cast<int>(packet.type)
                          << " | Data: " << packet.data << std::endl;
                break;
        }
    }
    
    std::cout << "Disconnected from server." << std::endl;
    closesocket(sock);
}



