#include <winsock2.h>
#include <ws2tcpip.h>
#include <iostream>
#include <string>
#include <thread>
#include <chrono>
#include "Server.h"
#include "Client.h"
#include "Utils.h"

int main(int argc, char* argv[]) {
    std::cout << "[Server] Starting Server on port:" << argv[2] << "" << std::endl;

    WSADATA wsaData;
    if(WSAStartup(MAKEWORD(2,2), &wsaData) != 0) {
        std::cerr << "WSAStartup failed." << std::endl;
        return EXIT_FAILURE;
    }

    if(argc < 3) {
        std::cout << "Usage:\n  Server: " << argv[0] << " server <port>\n  Client: " << argv[0] << " client <IP>:<port>\n";
        WSACleanup();
        return EXIT_FAILURE;
    }

    std::string mode = argv[1];
    if(mode == "server") {
        unsigned short port = static_cast<unsigned short>(std::stoi(argv[2]));
        Server server(port);
        server.start();
        // Heartbeat thread: send a heartbeat with current timestamp every second.
        std::thread heartbeat([&server](){
            while(true) {
                std::this_thread::sleep_for(std::chrono::seconds(1));
                Packet heartbeat;
                heartbeat.type = HEARTBEAT;
                heartbeat.data = getCurrentTimestamp();
                server.broadcast(heartbeat);
            }
        });


        while(true) {
            std::this_thread::sleep_for(std::chrono::seconds(1));
        }
        heartbeat.join();
    } else if(mode == "client") {

        std::string ip_port = argv[2];
        size_t colonPos = ip_port.find(':');
        if(colonPos == std::string::npos) {
            std::cerr << "Invalid client format. Use IP:port" << std::endl;
            WSACleanup();
            return EXIT_FAILURE;
        }
        std::string ip = ip_port.substr(0, colonPos);
        std::string port = ip_port.substr(colonPos + 1);
        runClient(ip, port);

    } else {

        std::cerr << "Unknown mode: " << mode << std::endl;
        WSACleanup();
        return EXIT_FAILURE;

    }

    WSACleanup();
    
    return EXIT_SUCCESS;

}
